@@ -3,15 +3,21 @@ from __future__ import annotations
 
 import argparse
 import json
-import logging
 import os
 from typing import Any
 
-from .constants import FIND_DASHBOARD_POST, FIND_QUERY_POST
 from .api import DuneAPI
+from .constants import FIND_DASHBOARD_POST, FIND_QUERY_POST
+from .logger import set_log
 from .types import DuneQuery, DashboardTile, Post, Network, QueryParameter
+from .util import duplicates
 
 BASE_URL = "https://dune.xyz"
+log = set_log(__name__)
+
+
+class DuplicateQueryError(Exception):
+    """Basic extension of Exception class"""
 
 
 class DuneDashboard:
@@ -26,18 +32,36 @@ class DuneDashboard:
     queries: list[DuneQuery]
     api: DuneAPI
 
-    def __init__(self, api: DuneAPI, name: str, user: str, queries: set[DuneQuery]):
-        # Tile Validation
-        assert len(set(q.raw_sql for q in queries)) == len(queries), "Duplicate query"
+    def __init__(
+        self, api: DuneAPI, name: str, slug: str, user: str, queries: list[DuneQuery]
+    ):
+        dupes = duplicates([(q.raw_sql, q.network) for q in queries])
+        if dupes:
+            log.warning(f"Duplicate Query Detected {dupes}")
+            # raise DuplicateQueryError(dupes)
+
         if api.username != user:
             raise ValueError(
                 f"Attempt to load dashboard queries for invalid user {user} != {api.username}."
             )
         self.name = name
-        self.url = "/".join([BASE_URL, user, name])
+        self.slug = slug
+        self.url = "/".join([BASE_URL, user, slug])
         self.queries = list(queries)
         self.api = api
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, DuneDashboard):
+            return NotImplemented
+        equality_conditions = [
+            self.name == other.name,
+            self.slug == other.slug,
+            self.url == other.url,
+            self.queries == other.queries,
+        ]
+        log.debug(f"Equality conditions: {equality_conditions}")
+        return all(equality_conditions)
+
     @classmethod
     def from_file(cls, api: DuneAPI, filename: str) -> DuneDashboard:
         """Constructs Dashboard from configuration file"""
@@ -61,7 +85,7 @@ class DuneDashboard:
             },
             "query": FIND_DASHBOARD_POST,
         }
-
+        print(dashboard_slug)
         response = api.post_dune_request(
             Post(data=post_data, key_map={"dashboards": {"visualization_widgets"}})
         )
@@ -82,10 +106,12 @@ class DuneDashboard:
             )
             response = api.post_dune_request(post)
             query_data = response.json()["data"]["queries"][0]
+            # Filtering out queries that are not owned by logged-in user.
             if query_data["user"]["name"] == api.username:
                 queries.add(
                     DuneQuery(
                         name=query_data["name"],
+                        description=query_data["description"],
                         raw_sql=query_data["query"],
                         network=Network(query_data["dataset_id"]),
                         parameters=[
@@ -96,7 +122,7 @@ class DuneDashboard:
                     )
                 )
             else:
-                logging.info(
+                log.info(
                     f'Ignoring dashboard query from user {query_data["user"]["name"]}'
                 )
 
@@ -108,12 +134,18 @@ class DuneDashboard:
                 name=meta["name"],
                 owner=dashboard_owner,
                 slug=dashboard_slug,
-                queries=queries,
+                queries=list(queries),
             )
-        return cls(api=api, name=meta["name"], queries=queries, user=dashboard_owner)
+        return cls(
+            api=api,
+            name=meta["name"],
+            slug=dashboard_slug,
+            queries=list(queries),
+            user=dashboard_owner,
+        )
 
     @staticmethod
-    def dump_config(name: str, owner: str, slug: str, queries: set[DuneQuery]) -> None:
+    def dump_config(name: str, owner: str, slug: str, queries: list[DuneQuery]) -> None:
         """
         Writes Dashboard Configuration to files.
         Specifically to ./out/Dashboard-Slug
@@ -121,41 +153,63 @@ class DuneDashboard:
         out_dir = f"./out/{slug}"
         if not os.path.exists(out_dir):
             os.makedirs(out_dir)
-        with open(f"{out_dir}/config.json", "w", encoding="utf-8") as config_file:
+        queries_seen = {}
+        with open(f"{out_dir}/_config.json", "w", encoding="utf-8") as config_file:
             query_dicts = []
             for query in queries:
-                query_file = f"{out_dir}/{query.name.lower().replace(' ', '-')}.sql"
-                query_dicts.append(
-                    {
-                        "id": query.query_id,
-                        "name": query.name,
-                        "query_file": query_file,
-                        "network": str(query.network),
-                        "parameters": [
-                            {"key": p.key, "value": p.value, "type": p.type.value}
-                            for p in query.parameters
-                        ],
-                    }
-                )
-                with open(query_file, "w", encoding="utf-8") as q_file:
-                    q_file.write(query.raw_sql)
+                query_file = f"{query.name.lower().replace(' ', '-')}.sql"
+                query_config = {
+                    "id": query.query_id,
+                    "name": query.name,
+                    "description": query.description,
+                    "query_file": query_file,
+                    "network": str(query.network),
+                    "parameters": [
+                        {"key": p.key, "value": p.value, "type": p.type.value}
+                        for p in query.parameters
+                    ],
+                }
+                query_dicts.append(query_config)
+
+                if query.raw_sql not in queries_seen:
+                    # Whenever we encounter a new SQL query, write to file
+                    query_path = f"{out_dir}/{query_file}"
+                    with open(query_path, "w", encoding="utf-8") as q_file:
+                        q_file.write(query.raw_sql.strip("\n") + "\n")
+                    queries_seen[query.raw_sql] = query_file
+                else:
+                    # If already seen use the already existing file.
+                    query_config["query_file"] = queries_seen[query.raw_sql]
 
             config_dict = {
                 "meta": {
+                    # Dashboards can be renamed but the slug doesn't change.
                     "name": name,
+                    "slug": slug,
                     "user": owner,
+                    "query_path": out_dir,
                 },
                 "queries": query_dicts,
             }
-            config_file.write(json.dumps(config_dict))
+            config_file.write(json.dumps(config_dict, indent=2).strip("\n") + "\n")
 
     @classmethod
     def from_json(cls, api: DuneAPI, json_obj: dict[str, Any]) -> DuneDashboard:
         """Constructs Dashboard from json file"""
         meta, queries = json_obj["meta"], json_obj["queries"]
-        tiles = [DashboardTile.from_dict(q) for q in queries]
-        queries = {DuneQuery.from_tile(tile) for tile in tiles}
-        return cls(api=api, name=meta["name"], user=meta["user"], queries=queries)
+        # TODO - tiles could be phased out of this program.
+        tiles = [DashboardTile.from_dict(q, meta["query_path"]) for q in queries]
+        queries = [DuneQuery.from_tile(tile) for tile in tiles]
+        name = meta["name"]
+        return cls(
+            api=api,
+            name=name,
+            # Dashboards can be renamed, but slug is permanent.
+            # So we chose slug with priority and use name otherwise.
+            slug=meta.get("slug", name.replace(" ", "-")),
+            user=meta["user"],
+            queries=queries,
+        )
 
     def update(self) -> None:
         """Creates a dune connection and updates/refreshes all dashboard queries"""
