@@ -6,9 +6,9 @@ All operations/routes available for interaction with Dune API - looks like graph
 """
 from __future__ import annotations
 
-import re
 import json
 import os
+import re
 from dataclasses import dataclass
 from datetime import datetime
 from enum import Enum
@@ -16,8 +16,11 @@ from typing import Any, Collection, Optional
 
 from dotenv import load_dotenv
 
+from .logger import set_log
 from .util import datetime_parser, open_query, postgres_date
 
+log = set_log(__name__)
+
 PostData = dict[str, Collection[str]]
 # key_map = {"outer1": {"inner11", "inner12}, "outer2": {"inner21"}}
 KeyMap = dict[str, set[str]]
@@ -42,7 +45,10 @@ class MetaData:
 
     def __init__(self, obj: str):
         """
+        Constructor method
         :param obj: input should have the following form
+
+        Example input:
         {
             'id': '3158cc2c-5ed1-4779-b523-eeb9c3b34b21',
             'job_id': '093e440d-66ce-4c00-81ec-2406f0403bc0',
@@ -168,6 +174,17 @@ class QueryParameter:
         self.type: ParameterType = parameter_type
         self.value = value
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, QueryParameter):
+            return NotImplemented
+        return all(
+            [
+                self.key == other.key,
+                self.value == other.value,
+                self.type.value == other.type.value,
+            ]
+        )
+
     @classmethod
     def text_type(cls, name: str, value: str) -> QueryParameter:
         """Constructs a Query parameter of type text"""
@@ -229,7 +246,10 @@ class QueryParameter:
         raise ValueError(f"Could not parse Query parameter from {obj}")
 
     def __str__(self) -> str:
-        return f"QueryParameter(name: {self.key}, value: {self.value}, type: {self.type.value})"
+        return f"QueryParameter(" \
+               f"name: {self.key}, " \
+               f"value: {self.value}, " \
+               f"type: {self.type.value})"
 
 
 @dataclass
@@ -248,6 +268,7 @@ class DashboardTile:
     """
 
     name: str
+    description: str
     select_file: str
     query_id: int
     network: Network
@@ -255,11 +276,12 @@ class DashboardTile:
     base_file: Optional[str]
 
     @classmethod
-    def from_dict(cls, obj: dict[str, Any]) -> DashboardTile:
+    def from_dict(cls, obj: dict[str, Any], path: str) -> DashboardTile:
         """Constructs Record from Dune Data as string dict"""
         return cls(
             name=obj.get("name", "untitled"),
-            select_file=obj["query_file"],
+            description=obj.get("description", ""),
+            select_file="/".join([path, obj["query_file"]]),
             network=Network.from_string(obj["network"]),
             query_id=int(obj["id"]),
             parameters=[QueryParameter.from_dict(p) for p in obj.get("parameters", [])],
@@ -279,6 +301,7 @@ class DuneQuery:
     """Contains all the relevant data necessary to initiate a Dune Query"""
 
     name: str
+    description: str
     raw_sql: str
     network: Network
     parameters: list[QueryParameter]
@@ -287,10 +310,25 @@ class DuneQuery:
     def __hash__(self) -> int:
         return hash(self.query_id)
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, DuneQuery):
+            return NotImplemented
+        equality_conditions = [
+            self.name == other.name,
+            self.description == other.description,
+            self.raw_sql == other.raw_sql,
+            self.network.value == other.network.value,
+            self.query_id == other.query_id,
+            self.parameters == other.parameters,
+        ]
+        log.debug(f"Equality Conditions: {equality_conditions}")
+        return all(equality_conditions)
+
     @classmethod
     def from_environment(
         cls,
         raw_sql: str,
+        description: str,
         network: Network,
         parameters: Optional[list[QueryParameter]] = None,
         name: Optional[str] = None,
@@ -299,6 +337,7 @@ class DuneQuery:
         load_dotenv()
         return cls(
             raw_sql=raw_sql,
+            description=description,
             network=network,
             parameters=parameters if parameters is not None else [],
             name=name if name else "untitled",
@@ -310,6 +349,7 @@ class DuneQuery:
         """Constructs Dune Query from DashboardTile object"""
         return cls(
             name=tile.name,
+            description=tile.description,
             raw_sql=tile.build_query(),
             network=tile.network,
             parameters=tile.parameters,
@@ -328,7 +368,7 @@ class DuneQuery:
             "name": self.name,
             "query": self.raw_sql,
             "user_id": 84,
-            "description": "",
+            "description": self.description,
             "is_archived": False,
             "is_temp": False,
             "tags": [],
